@@ -34,6 +34,9 @@
 /* Define the Dosemu version */
 #undef VERSION
 
+/* Define the Dosemu version release date */
+#undef VERDATE
+
 /* Define the Dosemu sublevel */
 #undef SUBLEVEL
 
@@ -86,6 +89,9 @@
 /* Add if you have X-Keycode problems */
 #undef NEW_KEYCODES
 
+/* Define this to use the new keyboard code */
+#undef NEW_KBD_CODE
+
 /*
  *  DEFINE this, if you have joystick support in the kernel
  *  or have the joystick module loaded (needing <linux/joystick.h>)
@@ -101,7 +107,7 @@
 /* Define this if you want to use MITSHM */
 #undef HAVE_MITSHM
 
-/* Define this if you want to use a monoton micro timeing (Bernd Paysan) */
+/* Define this if you want to use a monoton micro timing (Bernd Paysan) */
 #undef MONOTON_MICRO_TIMING
 
 @BOTTOM@
