@@ -4,6 +4,7 @@
 #include <stdlib.h>
 #include <signal.h>
 #include <string.h>
+#include <errno.h>
 
 #ifdef __linux__
 #include <syscall.h>
@@ -21,9 +22,12 @@
 #include "ipx.h"
 #include "pktdrvr.h"
 
-extern void keyb_server_run(void);
+#ifdef NEW_KBD_CODE
+#include "keyb_clients.h"
+#endif
 
 extern void keyb_server_run(void);
+extern void irq_select(void);
 
 #ifdef __NetBSD__
 extern int errno;
@@ -43,6 +47,40 @@ static struct SIGNAL_queue signal_queue[MAX_SIG_QUEUE_SIZE];
 static sigset_t oldset;
 
 #ifdef __linux__
+#if __GLIBC__ > 1		/* glibc-2 version */
+/*
+ * Thomas Winder <thomas.winder@sea.ericsson.se> wrote:
+ * glibc-2 uses a different struct sigaction type than the one used in
+ * the kernel. The function dosemu_sigaction (in
+ * src/arch/linux/async/signal.c) bypasses the glibc provided syscall
+ * by doint the int 0x80 by hand. This call expects struct sigaction
+ * types as defined by the kernel. The whole story ends up with an
+ * incorrect set sa_restorer field, which yields a wrong stack when
+ * handling signals occuring when dpmi is active, which eventually
+ * segfaults the program.
+ */
+int
+dosemu_sigaction(int sig, struct sigaction *new, struct sigaction *old)
+{
+  struct my_sigaction {
+    __sighandler_t sa_handler;
+    unsigned long sa_mask;
+    unsigned long sa_flags;
+    void (*sa_restorer)(void);
+  };
+
+  struct my_sigaction my_sa;
+
+  my_sa.sa_handler = new->sa_handler;
+  my_sa.sa_mask = *((unsigned long *) &(new->sa_mask));
+  my_sa.sa_flags = new->sa_flags;
+  my_sa.sa_restorer = new->sa_restorer;
+
+  return(syscall(SYS_sigaction, sig, &my_sa, NULL));
+}
+
+#else		/* libc5 version */
+
 /* Similar to the sigaction function in libc, except it leaves alone the
    restorer field
    stolen from the wine-project */
@@ -56,7 +94,8 @@ dosemu_sigaction(int sig, struct sigaction *new, struct sigaction *old)
   errno = -sig;
   return -1;
 }
-#endif
+#endif /* not __GLIBC__ */
+#endif /* __linux__ */
 
 /* DANG_BEGIN_FUNCTION signal_init
  *
@@ -191,6 +230,27 @@ void handle_signals(void) {
   }
 }
 
+/* ==============================================================
+ *
+ * This is called by default at around 100Hz.
+ * (see timer_interrupt_init() in init.c)
+ *
+ * The actual formulas, starting with the configurable parameter
+ * config.freq, are:
+ *	config.freq				default=18
+ *	config.update = 1E6/config.freq		default=54945
+ *	timer tick(us) = config.update/6	default=9157.5us
+ *		       = 166667/config.freq
+ *	timer tick(Hz) = 6*config.freq		default=100Hz
+ *
+ * 6 is the magical TIMER_DIVISOR macro used to get 100Hz
+ *
+ * This call should NOT be used if you need timing accuracy - many
+ * signals can get lost e.g. when kernel accesses disk, and the whole
+ * idea of timing-by-counting is plain wrong. We'll need the Pentium
+ * counter here.
+ * ============================================================== */
+
 void SIGALRM_call(void){
 
   static volatile int running = 0;
@@ -205,9 +265,26 @@ void SIGALRM_call(void){
 #endif
 
 #ifdef X_SUPPORT
-  if (config.X) 
+  if (config.X) {
      X_handle_events();
+#ifdef NEW_KBD_CODE
+     /* although actually the event handler handles the keyboard in X, keyb_client_run
+      * still needs to be called in order to handle pasting.
+      */
+     keyb_client_run();
 #endif
+  }
+#endif
+
+#ifdef NEW_KBD_CODE
+  /* for other front-ends, keyb_client_run() is called from ioctl.c if data is
+   * available, so we don't need to do it here.
+   */
+  
+  keyb_server_run();
+#endif
+   
+
 
 #if 1
 #ifdef USING_NET
@@ -307,9 +384,11 @@ void SIGALRM_call(void){
   if (not_use_sigio)
     io_select(fds_no_sigio);
 
-  /* this is for per-second activities */
+  /* this should be for per-second activities, it is actually at
+   * 180ms more or less */
   partials++;
-  if (partials == FREQ) {
+  /* if you want a REAL second here use: config.freq*TIMER_DIVISOR */
+  if (partials == config.freq) {
     partials = 0;
 #ifdef IPX
   if (config.ipxsup)
