@@ -53,7 +53,11 @@
 #define MHP_PRIVATE
 #include "mhpdbg.h"
 
-#define MAP_LOC		"/usr/src/dosemu/bin/dosemu.map"
+#ifndef MAP_LOC
+  #define MAP_LOC		"/usr/src/dosemu/bin/dosemu.map"
+#endif
+
+char dosemu_map_file_name[256] = MAP_LOC;
 
 #define makeaddr(x,y) ((((unsigned long)x) << 4) + (unsigned long)y)
 
@@ -358,19 +362,23 @@ static void mhp_rmapfile(int argc, char *argv[])
   unsigned char bytebuf[IBUFS];
   unsigned long a1;
 
-  ifp = fopen(MAP_LOC, "r");
+  ifp = fopen(dosemu_map_file_name, "r");
   if (!ifp) {
      mhp_printf("unable to open map file\n");
      return;
   }
   mhp_printf("Reading map file\n");
   last_symbol = 0;
-  for(;;) {
+  while (last_symbol < MAXSYM) {
      if(!fgets(bytebuf, 100, ifp))
         break;
-     if (!strlen(bytebuf))
+     if (!strlen(bytebuf) || !isxdigit(*bytebuf))
         continue;
-     sscanf(&bytebuf[0], "%lx", &a1);
+      /* recent versions of nm put out the address in long long
+       * format, with 16 digits: we can't rely on absolute offsets.
+       */
+      sscanf(bytebuf, "%lx %c %40s", &a1, &symbol_table[last_symbol].type,
+              (char *)&symbol_table[last_symbol].name);
 #ifdef __ELF__
      if (a1 < 0x08000000) continue;
 #else
@@ -380,8 +388,6 @@ static void mhp_rmapfile(int argc, char *argv[])
         break;
 #endif
      symbol_table[last_symbol].addr = a1;
-     symbol_table[last_symbol].type = bytebuf[9];
-     sscanf(&bytebuf[11], "%s", (char *)&symbol_table[last_symbol].name);
      last_symbol++;
   }
   fclose(ifp);
@@ -398,16 +404,22 @@ enum {
 
 static int decode_symreg(char *regn)
 {
-  static char reg_syms[]="SS  CS  DS  ES  FS  GS EAX EBX ECX EDX ESI EDI EBP ESP EIP  FL";
+  static char reg_syms[]="SS  CS  DS  ES  FS  GS  "
+			 "AX  BX  CX  DX  SI  DI  BP  SP  IP  FL  "
+ 			 "EAX EBX ECX EDX ESI EDI EBP ESP EIP ";
   char rn[5], *s;
   int n;
-  strncpy(rn,regn,4);
-  if (isalpha(rn[2])) rn[3]=0;
-  else rn[2]=0;
-  { char *c=rn; while (*c) *c++ = toupper(*c); }
-  if (!(s=strstr(reg_syms,rn))) return -1;
-  n=s-reg_syms;
-  return (n >> 2) + ((n & 1) ? (16-5) : 0);
+
+  if (!isalpha(*regn))
+	return -1;
+  s=rn; n=0; rn[4]=0;
+  while (n<4)
+  {
+	*s++=(isalpha(*regn)? toupper(*regn++): ' '); n++;
+  }
+  if (!(s = strstr(reg_syms, rn)))
+	return -1;
+  return ((s-reg_syms) >> 2);
 }
 
 static unsigned long mhp_getreg(unsigned char * regn)
