@@ -1,12 +1,10 @@
-/*
- * $Id: config.c,v 1.6 1995/05/06 16:26:13 root Exp root $
- */
 #include <stdio.h>
 #include <termios.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
+#include <errno.h>
 
 #include "config.h"
 #include "emu.h"
@@ -19,10 +17,8 @@
 #include "kversion.h"
 
 #include "dos2linux.h"
+#include "priv.h"
 
-#ifdef __NetBSD__
-extern int errno;
-#endif
 
 /*
  * XXX - the mem size of 734 is much more dangerous than 704. 704 is the
@@ -105,12 +101,17 @@ config_defaults(void)
     config.dualmon = 0;
     config.force_vt_switch = 0;
     config.speaker = SPKR_EMULATED;
-#if 0				/* This is too slow, but why? */
-    config.update = 54945;
-#else
-    config.update = 27472;
-#endif
-    config.freq = 18;		/* rough frequency */
+
+    /* The frequency in usec of the SIGALRM call (in signal.c) is
+     * equal to this value / 6, and thus is currently 9158us = 100 Hz
+     * The 6 (TIMER DIVISOR) is a constant of unknown origin
+     * NOTE: if you set 'timer 18' in config.dist you can't get anything
+     * better that 55555 (108Hz) because of integer math.
+     * see timer_interrupt_init() in init.c
+     */
+    config.update = 54945;	/* should be = 1E6/config.freq */
+    config.freq = 18;		/* rough frequency (real PC = 18.2065) */
+
     config.timers = 1;		/* deliver timer ints */
     config.keybint = 1;		/* keyboard interrupts */
  
@@ -170,7 +171,9 @@ config_defaults(void)
 static void 
 open_terminal_pipe(char *path)
 {
+    priv_off();
     terminal_fd = DOS_SYSCALL(open(path, O_RDWR));
+    priv_default();
     if (terminal_fd == -1) {
 	terminal_pipe = 0;
 	error("ERROR: open_terminal_pipe failed - cannot open %s!\n", path);
@@ -252,7 +255,7 @@ config_init(int argc, char **argv)
 		  fprintf(stderr, "Sorry, no access to configuration file %s\n", optarg);
 		  exit(1);
 		}
-		close(f);
+		fclose(f);
 	    }
 	    confname = optarg;	/* someone reassure me that this is *safe*? */
 	    break;
@@ -269,14 +272,14 @@ config_init(int argc, char **argv)
 	    dbg_fd = stderr;
 	    break;
 	case 'o':
-	    priv_off();
 	    config.debugout = strdup(optarg);
+	    priv_off();
 	    dbg_fd = fopen(config.debugout, "w");
+	    priv_default();
 	    if (!dbg_fd) {
 		fprintf(stderr, "can't open \"%s\" for writing\n", config.debugout);
 		exit(1);
 	    }
-	    priv_default();
 	    break;
 	}
     }
@@ -288,12 +291,8 @@ config_init(int argc, char **argv)
     }
 #endif
 
-    priv_off();
-
     parse_config(confname);
 
-    priv_default();
-
     if (config.exitearly)
 	leavedos(0);
 
@@ -335,6 +334,7 @@ config_init(int argc, char **argv)
 #endif
 	    break;
 	case 'Y':
+#ifdef X_SUPPORT
 	    open_Xkeyboard_pipe(optarg);
 	    config.cardtype = CARD_MDA;
 	    config.mapped_bios = 0;
@@ -348,10 +348,13 @@ config_init(int argc, char **argv)
 	    config.vga = 0;	/* this flags BIOS graphics */
 	    config.usesX = 1;
 	    config.console_keyb = 1;
+#endif
 	    break;
 	case 'Z':
+#ifdef X_SUPPORT
 	    open_Xmouse_pipe(optarg);
 	    config.usesX = 1;
+#endif
 	    break;
 	case 'K':
 	    warn("Keyboard interrupt enabled...this is still buggy!\n");
@@ -369,9 +372,7 @@ config_init(int argc, char **argv)
 	    break;
 	case 'P':
 	    if (terminal_fd == -1) {
-		priv_off();
 		open_terminal_pipe(optarg);
-		priv_default();
 	    } else
 		error("ERROR: terminal pipe already open\n");
 	    break;
@@ -384,7 +385,7 @@ config_init(int argc, char **argv)
 	    break;
 	case 'v':
 	    config.cardtype = atoi(optarg);
-	    if (config.cardtype > 4)
+	    if (config.cardtype > 7)	/* keep it updated when adding a new card! */
 		config.cardtype = 1;
 	    g_printf("Configuring cardtype as %d\n", config.cardtype);
 	    break;
@@ -645,39 +646,46 @@ int parse_debugflags(const char *s, unsigned char flag)
 static void
 usage(void)
 {
-    fprintf(stdout, "$Header: /usr/src/dosemu0.60/init/RCS/config.c,v 1.6 1995/05/06 16:26:13 root Exp root $\n");
+    fprintf(stdout, "dosemu 0.66\n");
     fprintf(stdout, "usage: dos [-ABCckbVNtsgxKm234e] [-D flags] [-M SIZE] [-P FILE] [ -F File ] 2> dosdbg\n");
+    fprintf(stdout, "    -2,3,4,5 choose 286, 386, 486 or 586 CPU\n");
     fprintf(stdout, "    -A boot from first defined floppy disk (A)\n");
     fprintf(stdout, "    -B boot from second defined floppy disk (B) (#)\n");
+    fprintf(stdout, "    -b map BIOS into emulator RAM (%%)\n");
     fprintf(stdout, "    -C boot from first defined hard disk (C)\n");
     fprintf(stdout, "    -c use PC console video (!%%)\n");
-    fprintf(stdout, "    -k use PC console keyboard (!)\n");
+    fprintf(stdout, "    -d detach (?)\n");
 #ifdef X_SUPPORT
     fprintf(stdout, "    -X run in X Window (#)\n");
 /* seems no longer valid bo 18.7.95
-    fprintf(stdout, "    -X NAME use MDA direct and FIFO NAME for keyboard (only with x2dos!)\n");
-    fprintf(stdout, "    -Y NAME use FIFO NAME for mouse (only with x2dos!)\n");
+    fprintf(stdout, "    -Y NAME use MDA direct and FIFO NAME for keyboard (only with x2dos!)\n");
+    fprintf(stdout, "    -Z NAME use FIFO NAME for mouse (only with x2dos!)\n");
 */
-    fprintf(stdout, "    -D set debug-msg mask to flags (+-)(dRWDCvXkism#pgcwhIExMnPrS01)\n");
+    fprintf(stdout, "    -D set debug-msg mask to flags {+-}{0-9}{#CDEIMPRSWXcdghikmnprsvwx}\n");
 #else				/* X_SUPPORT */
-    fprintf(stdout, "    -D set debug-msg mask to flags (+-)(dRWDCvkism#pgcwhIExMnPrS01)\n");
+    fprintf(stdout, "    -D set debug-msg mask to flags {+-}{0-9}{#CDEIMPRSWcdghikmnprsvwx}\n");
 #endif				/* X_SUPPORT */
-    fprintf(stdout, "    -M set memory size to SIZE kilobytes (!)\n");
-    fprintf(stdout, "    -P copy debugging output to FILE\n");
-    fprintf(stdout, "    -b map BIOS into emulator RAM (%%)\n");
-    fprintf(stdout, "    -F use config-file File\n");
-    fprintf(stdout, "    -V use BIOS-VGA video modes (!#%%)\n");
-    fprintf(stdout, "    -N No boot of DOS\n");
-    fprintf(stdout, "    -t try new timer code (#)\n");
-    fprintf(stdout, "    -s try new screen size code (#)\n");
-    fprintf(stdout, "    -g enable graphics modes (!%%#)\n");
-    fprintf(stdout, "    -x SIZE enable SIZE K XMS RAM\n");
+    fprintf(stdout, "       #=defint  C=cdrom    D=dos    E=ems       I=ipc     M=dpmi\n");
+    fprintf(stdout, "       P=packet  R=diskread S=sound  W=diskwrite c=config  d=disk\n");
+    fprintf(stdout, "       g=general h=hardware i=i/o    k=keyb      m=mouse   n=ipxnet\n");
+    fprintf(stdout, "       p=printer r=pic      s=serial v=video     w=warning x=xms\n");
+    fprintf(stdout, "    -E STRING pass DOS command on command line\n");
     fprintf(stdout, "    -e SIZE enable SIZE K EMS RAM\n");
+    fprintf(stdout, "    -F use config-file File\n");
+    fprintf(stdout, "    -g enable graphics modes (!%%#)\n");
+    fprintf(stdout, "    -K Do int9 (!#)\n");
+    fprintf(stdout, "    -k use PC console keyboard (!)\n");
+    fprintf(stdout, "    -M set memory size to SIZE kilobytes (!)\n");
     fprintf(stdout, "    -m enable mouse support (!#)\n");
-    fprintf(stdout, "    -o FILE put debugmessages in file\n");
+    fprintf(stdout, "    -N No boot of DOS\n");
     fprintf(stdout, "    -O write debugmessages to stderr\n");
-    fprintf(stdout, "    -2,3,4,5 choose 286, 386, 486 or 586 CPU\n");
-    fprintf(stdout, "    -K Do int9 (!#)\n\n");
+    fprintf(stdout, "    -o FILE put debugmessages in file\n");
+    fprintf(stdout, "    -P copy debugging output to FILE\n");
+    fprintf(stdout, "    -T DIR set tmpdir\n");
+    fprintf(stdout, "    -t try new timer code (#)\n");
+    fprintf(stdout, "    -V use BIOS-VGA video modes (!#%%)\n");
+    fprintf(stdout, "    -v NUM force video card type\n");
+    fprintf(stdout, "    -x SIZE enable SIZE K XMS RAM\n");
     fprintf(stdout, "    (!) BE CAREFUL! READ THE DOCS FIRST!\n");
     fprintf(stdout, "    (%%) require dos be run as root (i.e. suid)\n");
     fprintf(stdout, "    (#) options do not fully work yet\n");
