@@ -1,11 +1,6 @@
 /* dos emulator, Matthias Lautner
  * Extensions by Robert Sanders, 1992-93
  *
- * $Date: 1995/05/06 16:25:30 $
- * $Source: /usr/src/dosemu0.60/dosemu/RCS/disks.c,v $
- * $Revision: 2.11 $
- * $State: Exp $
- *
  * floppy disks, dos partitions or their images (files) (maximum 8 heads)
  */
 
@@ -29,6 +24,7 @@
 
 #include "emu.h"
 #include "disks.h"
+#include "priv.h"
 
 
 #define FDISKS config.fdisks
@@ -97,12 +93,24 @@ int
 read_sectors(struct disk *dp, char *buffer, long head, long sector,
 	     long track, long count)
 {
-  long pos;
+#ifdef __linux__
+  loff_t  pos;
+#else
+  off_t  pos;
+#endif
   long already = 0;
   long tmpread;
 
   /* XXX - header hack. dp->header can be negative for type PARTITION */
   pos = DISK_OFFSET(dp, head, sector, track) + dp->header;
+  d_printf("DISK: Trying to read %ld bytes at T/S/H %ld/%ld/%ld",
+	   count,track,sector,head);
+#ifdef __linux__
+  d_printf(" at pos %Ld\n", pos);
+#else
+  d_printf(" at pos %ld\n", pos);
+#endif
+
 
   /* reads beginning before that actual disk/file */
   if (pos < 0) {
@@ -137,9 +145,14 @@ read_sectors(struct disk *dp, char *buffer, long head, long sector,
       mbrcount = dp->part_info.mbr_size - mbroff;
     }
 
-    d_printf("WARNING: %ld (0x%lx) read for %d below %s, "
+#ifdef __linux__
+    d_printf("WARNING: %Ld",pos);
+#else
+    d_printf("WARNING: %ld",pos);
+#endif
+    d_printf(" (0x%lx) read for %d below %s, "
 	     "h:%ld s:%ld t:%ld\n",
-	     pos, (unsigned long) -pos, readsize,
+	     (unsigned long) -pos, readsize,
 	     dp->dev_name, head, sector, track);
 
     memcpy(buffer, dp->part_info.mbr + mbroff, mbrcount);
@@ -162,7 +175,11 @@ read_sectors(struct disk *dp, char *buffer, long head, long sector,
     }
   }
 
+#ifdef __linux__
+  if (pos != llseek(dp->fdesc, pos, SEEK_SET)) {
+#else
   if (pos != lseek(dp->fdesc, pos, SEEK_SET)) {
+#endif
     error("ERROR: Sector not found in read_sector, error = %s!\n",
 	  strerror(errno));
     return -DERR_NOTFOUND;
@@ -179,7 +196,11 @@ int
 write_sectors(struct disk *dp, char *buffer, long head, long sector,
 	      long track, long count)
 {
+#ifdef __linux__
+  loff_t  pos;
+#else
   off_t pos;
+#endif
   int tmpwrite;
 
   if (dp->rdonly) {
@@ -194,6 +215,13 @@ write_sectors(struct disk *dp, char *buffer, long head, long sector,
 
   /* XXX - header hack */
   pos = DISK_OFFSET(dp, head, sector, track) + dp->header;
+  d_printf("DISK: Trying to write %ld bytes T/S/H %ld/%ld/%ld",
+	   count,track,sector,head);
+#ifdef __linux__
+  d_printf(" at pos %Ld\n", pos);
+#else
+  d_printf(" at pos %ld\n", pos);
+#endif
 
   /* if dp->type is PARTITION, we currently don't allow writing to an area
    * not within the actual partition (i.e. somewhere else on the faked
@@ -204,7 +232,11 @@ write_sectors(struct disk *dp, char *buffer, long head, long sector,
    * Also, I could allow writing to the fake MBR, but I'm not sure that's
    * a good idea.
    */
+#ifdef __linux__
+  if (pos != llseek(dp->fdesc, pos, SEEK_SET)) {
+#else
   if (pos != lseek(dp->fdesc, pos, SEEK_SET)) {
+#endif
     error("ERROR: Sector not found in write_sector!\n");
     return -DERR_NOTFOUND;
   }
@@ -252,7 +284,8 @@ image_auto(struct disk *dp)
     }
   }
 
-  lseek(dp->fdesc, 0, SEEK_SET);
+  lseek(dp->fdesc, 0, SEEK_SET);/* Use lseek here, as nobody want
+				   > 2^^31 bytes for the image file */
   if (RPT_SYSCALL(read(dp->fdesc, header, HEADER_SIZE)) != HEADER_SIZE) {
     error("ERROR: could not read full header in image_init\n");
     leavedos(19);
@@ -351,7 +384,10 @@ partition_setup(struct disk *dp)
   hd_name[8] = '\0';			/* i.e.  /dev/hda6 -> /dev/hda */
 #endif
 
-  if ((part_fd = DOS_SYSCALL(open(hd_name, O_RDONLY))) == -1) {
+  priv_on();
+  part_fd = DOS_SYSCALL(open(hd_name, O_RDONLY));
+  priv_default();
+  if (part_fd == -1) {
     error("ERROR: opening device %s to read MBR for PARTITION %s\n",
 	  hd_name, dp->dev_name);
     leavedos(22);
@@ -511,10 +547,14 @@ disk_open(struct disk *dp)
   if (dp == NULL || dp->fdesc >= 0)
     return;
     
+  priv_on();
   dp->fdesc = DOS_SYSCALL(open(dp->dev_name, dp->wantrdonly ? O_RDONLY : O_RDWR, 0));
+  priv_default();
   if (dp->fdesc < 0) 
     if (errno == EROFS || errno == EACCES) {
+      priv_on();
       dp->fdesc = DOS_SYSCALL(open(dp->dev_name, O_RDONLY, 0));
+      priv_default();
       if (dp->fdesc < 0) {
         d_printf("ERROR: (disk) can't open %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
         /* In case we DO get more clever, we want to share that code */
@@ -578,10 +618,15 @@ disk_open(struct disk *dp)
   if (dp == NULL || dp->fdesc >= 0)
     return;
     
+  priv_on();
   dp->fdesc = DOS_SYSCALL(open(dp->dev_name, dp->wantrdonly ? O_RDONLY : O_RDWR, 0));
+  priv_default();
+
   if (dp->fdesc < 0) 
     if (errno == EROFS) {
+      priv_on();
       dp->fdesc = DOS_SYSCALL(open(dp->dev_name, O_RDONLY, 0));
+      priv_default();
       if (dp->fdesc < 0) {
         d_printf("ERROR: (disk) can't open %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
         /* In case we DO get more clever, we want to share that code */
@@ -684,11 +729,15 @@ disk_init(void)
 #endif
 
   if (config.bootdisk) {
+    priv_on();
     bootdisk.fdesc = open(bootdisk.dev_name,
 			  bootdisk.rdonly ? O_RDONLY : O_RDWR, 0);
+    priv_default();
     if (bootdisk.fdesc < 0) 
       if (errno == EROFS) {
+        priv_on();
         bootdisk.fdesc = open(bootdisk.dev_name, O_RDONLY, 0);
+        priv_default();
         if (bootdisk.fdesc < 0) {
           error("ERROR: can't open bootdisk %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
           leavedos(23);
@@ -730,10 +779,14 @@ disk_init(void)
       continue;
     }
 #endif
+    priv_on();
     dp->fdesc = open(dp->dev_name, dp->rdonly ? O_RDONLY : O_RDWR, 0);
+    priv_default();
     if (dp->fdesc < 0) 
       if (errno == EROFS || errno == EACCES) {
+        priv_on();
         dp->fdesc = open(dp->dev_name, O_RDONLY, 0);
+        priv_default();
         if (dp->fdesc < 0) {
           error("ERROR: can't open %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
           leavedos(25);
@@ -749,13 +802,17 @@ disk_init(void)
   }
   for (dp = hdisktab; dp < &hdisktab[HDISKS]; dp++) {
     if(dp->type == IMAGE)  {
-	priv_off();
 	d_printf("IMAGE: Using user permissions\n");
+	priv_off();
     }
+    else priv_on();
     dp->fdesc = open(dp->dev_name, dp->rdonly ? O_RDONLY : O_RDWR, 0);
+    priv_default();
     if (dp->fdesc < 0) {
       if (errno == EROFS || errno == EACCES) {
+        if (dp->type == IMAGE) priv_off(); else priv_on();
         dp->fdesc = open(dp->dev_name, O_RDONLY, 0);
+        priv_default();
         if (dp->fdesc < 0) {
           error("ERROR: can't open %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
           leavedos(26);
@@ -771,9 +828,6 @@ disk_init(void)
     else dp->rdonly = dp->wantrdonly;
     dp->removeable = 0;
 
-    if(dp->type == IMAGE)
-	priv_default();
-
     /* HACK: if unspecified geometry (-1) then try to get it from kernel.
        May only work on WD compatible disks (MFM/RLL/ESDI/IDE). */
     if (dp->sectors == -1)
@@ -851,7 +905,11 @@ int13(u_char i)
 {
   unsigned int disk, head, sect, track, number;
   int res;
-  long pos;
+#ifdef __linux__
+  loff_t  pos;
+#else
+  off_t  pos;
+#endif
   char *buffer;
   struct disk *dp;
 
@@ -861,6 +919,15 @@ int13(u_char i)
       dp = &bootdisk;
     else
       dp = &disktab[disk];
+    switch (HI(ax)) {
+      #define DISKETTE_MOTOR_TIMEOUT (*((unsigned char *)0x440))
+      /* NOTE: we don't need this counter, but older games seem to rely
+       * on it. We count it down in INT08 (bios.S) --SW, --Hans
+       */
+      case 0: case 2: case 3: case 5: case 10: case 11:
+        DISKETTE_MOTOR_TIMEOUT = 3*18;  /* set timout to 3 seconds */
+        break;
+    }
   }
   else if (disk >= 0x80 && disk < 0x80 + HDISKS)
     dp = &hdisktab[disk - 0x80];
@@ -1011,11 +1078,19 @@ int13(u_char i)
 		  dp->heads, dp->sectors, dp->tracks);
       break;
     }
+#ifdef __linux__
+    pos = (long long ) ((track * dp->heads + head) * dp->sectors + sect) << 9;
+#else
     pos = (long) ((track * dp->heads + head) * dp->sectors + sect) << 9;
+#endif
     /* XXX - header hack */
     pos += dp->header;
 
+#ifdef __linux__
+    if (pos != llseek(dp->fdesc, pos, 0)) {
+#else
     if (pos != lseek(dp->fdesc, pos, 0)) {
+#endif
       HI(ax) = DERR_NOTFOUND;
       REG(eflags) |= CF;
       error("ERROR: test: sector not found 6\n");
