@@ -1,13 +1,6 @@
 /* mouse.c for the DOS emulator
  *       Robert Sanders, gt8134b@prism.gatech.edu
  *
- * $Date: 1995/05/06 16:27:42 $
- * $Source: /usr/src/dosemu0.60/mouse/RCS/mouse.c,v $
- * $Revision: 2.20 $
- * $State: Exp $
- *
- * $Log: mouse.c,v $
- *
  */
 
 #include <stdio.h>
@@ -36,6 +29,13 @@
 
 #include "vc.h"
 #include "port.h"
+
+#ifdef NEW_KBD_CODE
+#include "keyboard.h"
+#endif
+
+#include "priv.h"
+
 extern int get_perm ();
 extern int release_perm ();
 
@@ -731,14 +731,14 @@ mouse_reset_to_current_video_mode(void)
   }
 
   /* virtual resolution is always at least 640x200 */
-  mouse.maxx = current_video.width;
   if (mouse.maxx < 640) {
 	mouse.maxx = 640;
+        mouse.maxy = current_video.height;
   }
-  mouse.maxy = current_video.height;
   if (mouse.maxy < 200) {
   	mouse.maxy = 200;
   	mouse.yshift = 3;
+	mouse.maxy = current_video.height << mouse.yshift;
   }
   else
   	mouse.yshift = 0;
@@ -1093,37 +1093,50 @@ void
 mouse_enable_internaldriver()
 {
   mice->intdrv = TRUE;
-  
   SETIVEC(0x33, Mouse_SEG, Mouse_INT);
   SETIVEC(0x74, Mouse_SEG, Mouse_INT74);
-
   m_printf("MOUSE: Enable InternalDriver\n");
 }
 
-void 
-mouse_keyboard(int sc)
+
+#ifndef NEW_KBD_CODE
+/* for the new keyboard code, these are defined in keyboard.h.
+ * The old keyboard code doesn't have symbolic constants, so we
+ * define them here
+ */
+#define KEY_LEFT  0x4b
+#define KEY_RIGHT 0x4d
+#define KEY_UP    0x48
+#define KEY_DOWN  0x50
+#define KEY_HOME  0x47
+#define KEY_END   0x4f
+#endif
+
+/* XXX - something's wrong here. Shouldn't the first 4 cases move the cursor?
+ */
+void mouse_keyboard(int sc)
 {
   switch (sc) {
-  case 0x50:
+  case KEY_DOWN:
     mouse.rbutton = mouse.lbutton = mouse.mbutton = 0;
     mouse_move();
     break;
-  case 0x4b:
+  case KEY_LEFT:
     mouse.rbutton = mouse.lbutton = mouse.mbutton = 0;
     mouse_move();
     break;
-  case 0x4d:
+  case KEY_RIGHT:
     mouse.rbutton = mouse.lbutton = mouse.mbutton = 0;
     mouse_move();
     break;
-  case 0x48:
+  case KEY_UP:
     mouse.rbutton = mouse.lbutton = mouse.mbutton = 0;
     mouse_move();
     break;
-  case 0x47:
+  case KEY_HOME:
     mouse_lb();
     break;
-  case 0x4f:
+  case KEY_END:
     mouse_rb();
     break;
   default:
@@ -1151,7 +1164,7 @@ mouse_round_coords()
 void 
 mouse_move(void)
 {
-
+ if (config.X) {
   get_current_video_mode();
   if (current_video.textgraph == 'T') {
     mouse.gfx_cursor = FALSE;
@@ -1178,7 +1191,7 @@ mouse_move(void)
   mouse.maxx = mouse_roundx(mouse.maxx - 1);
   mouse.maxy = mouse_roundy(mouse.maxy - 1);
   m_printf("maxx=%i, maxy=%i\n", mouse.maxx, mouse.maxy);
-
+ }
   if (mouse.x <= mouse.minx) mouse.x = mouse.minx;
   if (mouse.y <= mouse.miny) mouse.y = mouse.miny;
   if (mouse.x >= mouse.maxx) mouse.x = mouse.maxx;
@@ -1360,7 +1373,7 @@ mouse_event()
     m_printf("MOUSE: event %d, x %d ,y %d, mx %d, my %d, b %x\n",
 	     mouse_events, mouse.x, mouse.y, mouse.maxx, mouse.maxy, LWORD(ebx));
     m_printf("MOUSE: "
-	     "should call %04x:%04x (actually %04x:%04x)\n"
+	     "should call %04x:%04x (actually %04x:%04x)"
 	     ".........jumping to %04x:%04x\n",
 	     mouse.cs, mouse.ip, *mouse.csp, *mouse.ipp,
 	     LWORD(cs), LWORD(eip));	
@@ -1534,7 +1547,9 @@ mouse_init(void)
   if ( ! config.usesX ){
     if (mice->intdrv) {
       m_printf("Opening internal mouse: %s\n", mice->dev);
+      priv_on();  /* The mouse might need special permisions to open (esp r/w). */
       mice->fd = DOS_SYSCALL(open(mice->dev, O_RDWR | O_NONBLOCK));
+      priv_default();
       if (mice->fd == -1) {
  	mice->intdrv = FALSE;
  	mice->type = MOUSE_NONE;
@@ -1549,7 +1564,9 @@ mouse_init(void)
     }
 
     if ((mice->type == MOUSE_PS2) || (mice->type == MOUSE_BUSMOUSE)) {
+      priv_on();
       mice->fd = open(mice->dev, O_RDWR | O_NONBLOCK);
+      priv_default();
       mice->add_to_io_select = 0;
       if (mice->fd == -1) {
 	  error("Cannot open internal mouse device %s\n",mice->dev);
@@ -1608,13 +1625,14 @@ mouse_close(void)
     else
        m_printf("mouse_close: tcsetattr failed: %s\n",strerror(errno));
     m_printf("mouse_close: closing mouse device, fd=%d\n",mice->fd);
-    DOS_SYSCALL(close(mice->fd));
+    close(mice->fd);
     m_printf("mouse_close: ok\n");
     return;
   }
 
-  if ((mice->type == MOUSE_PS2) || (mice->type == MOUSE_BUSMOUSE))
-    DOS_SYSCALL(close(mice->fd));
+  if (((mice->type == MOUSE_PS2) || (mice->type == MOUSE_BUSMOUSE))
+	&& (mice->fd != -1))
+    close(mice->fd);
 }
 
 /* TO DO LIST: (in no particular order)
