@@ -5,21 +5,10 @@
  *
  * DANG_END_MODULE
  *
- * DANG_BEGIN_CHANGELOG
- *
  * much of this code originally written by Matthias Lautner
  * taken over by:
  *          Robert Sanders, gt8134b@prism.gatech.edu
  *
- * $Date: 1995/04/08 22:30:40 $
- * $Source: /usr/src/dosemu0.60/dosemu/RCS/cpu.c,v $
- * $Revision: 2.14 $
- * $State: Exp $
- *
- * $Log: cpu.c,v $
- *
- * DANG_END_CHANGELOG
- *
  */
 
 
@@ -40,6 +29,7 @@
 #include "vc.h"
 
 #include "../dpmi/dpmi.h"
+#include "priv.h"
 
 extern void xms_control(void);
 
