@@ -19,12 +19,6 @@
  * 
  * DANG_END_MODULE
  * 
- * DANG_BEGIN_CHANGELOG 
- * $Date: 1995/05/06 16:25:14 $ $Source: /home/src/dosemu0.60/RCS/emu.c,v $ $Revision: 2.38 $ $State: Exp $
- * 
- * $Log: emu.c,v $
- * 
- * DANG_END_CHANGELOG
  */
 
 
@@ -88,6 +82,7 @@ __asm__("___START___: jmp _emulate\n");
 #include <linux/vt.h>
 #include <linux/fd.h>
 #include <linux/hdreg.h>
+#include <linux/kd.h>
 #include <sys/vm86.h>
 #include <syscall.h>
 #endif
@@ -99,10 +94,6 @@ __asm__("___START___: jmp _emulate\n");
 #include "mhpdbg.h"
 #endif
 
-#ifdef REQUIRES_VM86PLUS
-  /* Please folks, don't remove this, it's required for emusys.h (within emu.h) */
-  #define __EMUSYS_parent
-#endif
 #include "emu.h"
 
 #include "bios.h"
@@ -117,15 +108,19 @@ __asm__("___START___: jmp _emulate\n");
 #include "hgc.h"
 #include "ipx.h"		/* TRB - add support for ipx */
 #include "serial.h"
-#include "keymaps.h"
 #include "int.h"
 #include "bitops.h"
 #include "pic.h"
 #include "dpmi.h"
+#include "priv.h"   /* for priv_init */
 #ifdef __NetBSD__
 #include <setjmp.h>
 #endif
 
+#ifdef NEW_KBD_CODE
+#include "keyb_clients.h"
+#endif
+
 extern void     stdio_init(void);
 extern void     time_setting_init(void);
 extern void     tmpdir_init(void);
@@ -134,7 +129,6 @@ extern void     low_mem_init(void);
 extern void     shared_memory_exit(void);
 extern void     restore_vt(u_short);
 extern void     disallocate_vt(void);
-extern void     keyboard_close(void);
 extern void     vm86_GP_fault();
 extern void     config_init(int argc, char **argv);
 extern void	timer_int_engine(void);
@@ -144,6 +138,11 @@ extern void io_select_init(void);
 
 static int      special_nowait = 0;
 
+#ifndef NEW_KBD_CODE
+extern void     keyboard_close(void);
+#endif
+
+
 void 
 boot(void)
 {
@@ -369,17 +368,8 @@ emulate(int argc, char **argv)
 #ifdef __NetBSD__
     changesegs();
 #endif
-    if (0 == geteuid()) {
-	warn("I am root\n");
-	i_am_root = 1;
-    } else {
-	if (getuid() != geteuid()) {
-	    fprintf(stderr, "Can't setuid to anyone else but root\n");
-	    exit(1);
-	}
-    }
 
-    priv_default();
+    priv_init();  /* This must come first! */
 
     /* the transposal of (config_|stdio_)init allows the addition of -o */
     /* to specify a debug out filename, if you're wondering */
@@ -445,7 +435,11 @@ dos_ctrl_alt_del(void)
     dbug_printf("DOS ctrl-alt-del requested.  Rebooting!\n");
     HMA_MAP(1);
     time_setting_init();
+#ifdef NEW_KBD_CODE   
+    keyb_server_reset();
+#else   
     keyboard_flags_init();
+#endif
     video_config_init();
     serial_init();
     mouse_init();
@@ -465,6 +459,7 @@ dos_ctrl_alt_del(void)
     timer_interrupt_init();	/* start sending int 8h int signals */
 }
 
+#if 0    /* this appears to be unused. */
 void
 dos_ctrlc(void)
 {
@@ -474,6 +469,7 @@ dos_ctrlc(void)
 
     do_soft_int(0x23);
 }
+#endif
 
 static void
 ign_sigs(int sig)
@@ -488,7 +484,7 @@ ign_sigs(int sig)
     else
 	otherints++;
 
-#define LEAVEDOS_TIMEOUT (3 * FREQ)
+#define LEAVEDOS_TIMEOUT (3 * config.freq)
 #define LEAVEDOS_SIGOUT  5
     if ((timerints >= LEAVEDOS_TIMEOUT) || (otherints >= LEAVEDOS_SIGOUT)) {
 	error("ERROR: timed/signalled out in leavedos()\n");
@@ -505,9 +501,19 @@ leavedos(int sig)
     extern int errno;
 
     static int recurse_check = 0;
-    if (recurse_check);
+   
+    warn("leavedos(%d) called - shutting down\n", sig);
+   
+    if (recurse_check)
+      {
+       error("leavedos called recursively, forgetting the graceful exit!\n");
+       flush_log();
+       _exit(sig);
+      }
     recurse_check = 1;
+#if 0
     priv_on();
+#endif
 #if 1 /* BUG CATCHER */
     if (in_vm86) {
       g_printf("\nkilled while in vm86(), trying to dump DOS-registers:\n");
@@ -531,6 +537,13 @@ leavedos(int sig)
     SETSIG(SIGTRAP, ign_sigs);
     warn("leavedos(%d) called - shutting down\n", sig);
 
+    if (config.speaker==SPKR_NATIVE ||
+       (config.speaker==SPKR_EMULATED && config.console && !config.X))
+    {
+       g_printf("SPEAKER: sound off\n");
+       do_ioctl(console_fd, KIOCSOUND, 0);	       /* turn off any sound */
+    }
+
     g_printf("calling close_all_printers\n");
     close_all_printers();
 
@@ -552,11 +565,19 @@ leavedos(int sig)
     disk_close_all();
     g_printf("calling video_close\n");
     video_close();
+   
     g_printf("calling keyboard_close\n");
+#ifdef NEW_KBD_CODE 
+    keyb_server_close();
+    keyb_client_close();
+
+
+#else
     fflush(stderr);
     fflush(stdout);
     keyboard_close();
-
+#endif
+   
     g_printf("calling shared memory exit\n");
     shared_memory_exit();
     g_printf("calling HMA exit\n");
@@ -569,6 +590,7 @@ leavedos(int sig)
 	restore_vt(config.detach);
 	disallocate_vt();
     }
+    flush_log();
     _exit(sig);
 }
 
