@@ -1,6 +1,5 @@
 /* for Linux DOS emulator
  *   Robert Sanders, gt8134b@prism.gatech.edu
- * $Id: disks.h,v 2.4 1995/04/08 22:35:19 root Exp $
  *
  */
 #ifndef DISKS_H
@@ -113,8 +112,13 @@ EXTERN struct disk bootdisk;
 EXTERN int use_bootdisk INIT(0);
 
 #if 1
+#ifdef __linux__
+#define DISK_OFFSET(dp,h,s,t) \
+  (((long long)(t * dp->heads + h) * dp->sectors + s) * SECTOR_SIZE)
+#else
 #define DISK_OFFSET(dp,h,s,t) \
   (((t * dp->heads + h) * dp->sectors + s) * SECTOR_SIZE)
+#endif
 #else
 #define DISK_OFFSET(dp,h,s,t) \
   (((h * dp->tracks + t) * dp->sectors + s) * SECTOR_SIZE)
